<?php
/**
 * Created by PhpStorm.
 * User: francismararaujo
 * Date: 23/10/14
 * Time: 17:07
 */

namespace TecDiet\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Method;

use TecDiet\Services\AppointmentsService;

class AppointmentsController extends \Zeedhi\Framework\Controller\Simple {

    protected $appointmentsService;

    public function __construct(AppointmentsService $appointmentsService) {
        $this->appointmentsService = $appointmentsService;
    }

    public function save(Request\DataSet $request, Response $response) {
        $appointments = $request->getDataSet()->getRows();
        if($appointments) {
            try {
                $dataSet = $this->appointmentsService->save($appointments);
                if(sizeof($dataSet)!=0){
                    $response->addDataSet(new DataSet('error', $dataSet));
                }
            } catch(\Exception $e) {
                $response->setError(new Error($e->getMessage(), $e->getCode()));
            };
        }
    }

    public function saveAnthEvaluationMethods(Request\DataSet $request, Response $response) {
        $anthEvaluationMethods = $request->getDataSet()->getRows();
        if($anthEvaluationMethods) {
            try {
                $dataSet = $this->appointmentsService->saveAnthEvaluationMethods($anthEvaluationMethods);
                $response->addDataSet(new DataSet('error', $dataSet));
            } catch(\Exception $e) {
                $response->setError(new Error($e->getMessage(), $e->getCode()));
            };
        }
    }

    public function saveDataInfoEvaluations(Request\DataSet $request, Response $response) {
        $dataInfoEvaluations = $request->getDataSet()->getRows();

        if($dataInfoEvaluations) {

            try {

                $this->appointmentsService->saveDataInfoEvaluations($dataInfoEvaluations);

            } catch(\Exception $e) {
                $response->setError(new Error($e->getMessage(), $e->getCode()));
            };

        }

    }

    public function getResult(Request\Row $request, Response $response) {
        try {
            $patient = $request->getRow();
            $result = $this->appointmentsService->getResult($patient);
            if(isset($result['ERROR'])){
                $response->addDataSet(new DataSet('error', $result));
            }else{
                $response->addDataSet(new DataSet ('Result', $result));
            }

        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        };

    }    

    public function saveDataAnthInfoEvaluations($request, $response) {

        try {
            
            $methods = $request->getDataSet()->getRows();

            $avaliations = array();

            for($i=0;$i<count($methods);$i++) {

                $method = $methods[$i];
                
                array_push($avaliations, $method['methodEvaluationId']);
                
            }
            
            $this->appointmentsService->saveDataInfoEvaluations($methods);

            $avaliations = implode(array_unique($avaliations),'_');
            
            $firstMethod = current($methods);
            
            $row = array(
                "id" => $firstMethod['id'],
                "unit" => $firstMethod['unit'],
                "appointmentDate" => $firstMethod['appointmentDate'],
                "avaliations" => $avaliations,
                "age" => $firstMethod['age']
            );

            $result = $this->appointmentsService->getResult($row);

            $response->addDataSet(new DataSet ('Result', $result));

        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        };

    }    

}