<?php

namespace TecDiet\Util;

use Doctrine\DBAL\DriverManager;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\Session\Session;
use TecDiet\Services\JWT;

class ConnectionFactory extends \Zeedhi\Framework\ORM\ConnectionFactory {
    
    public static function updateConnectionParams($connectionParams, $config, $eventManager){
	    $nrOrg = false;
	    
        $token = isset($_COOKIE["tecdietapp"]) ? $_COOKIE["tecdietapp"] : "";
        $session = empty($token) ? array() : JWT::getTokenData($token);
            //var_dump($session);
        if(isset($session) && count($session) > 0) {
            $nrOrg = isset($session['NRORG']) ? $session['NRORG'] : false;

            //Set current DBNAME in session for use in login session
            if(isset($connectionParams['sessionMode'])) {
                $encryptedDBName = hash('sha3-512', $connectionParams['dbname']);
                $encryptedDBName = hash('sha3-512', $connectionParams['dbname']);
                $session['CURRENT_DBNAME'] = preg_replace('/[^A-Za-z0-9\-]/', '', $encryptedDBName);
            } else {
                $encryptedUser = hash('sha3-512', $connectionParams['user']);
                $encryptedUser = hash('sha3-512', $connectionParams['user']);
                $session['CURRENT_DBNAME'] = preg_replace('/[^A-Za-z0-9\-]/', '', $encryptedUser);
            }
        }
         
        if ($nrOrg && isset($connectionParams['useVpd']) && $connectionParams['useVpd'] == 'true') {
            if(isset($connectionParams['vpdWithWallet']) && $connectionParams['vpdWithWallet'] == 'true') {
                unset($connectionParams['host']);
                unset($connectionParams['port']);
                $connectionParams['user'] = '/';
                $connectionParams['password'] = '';
                $connectionParams['dbname'] = 'USR_ORG_' . $nrOrg;
                $connectionParams['sessionMode'] = -2147483648;
            }else {
                $connectionParams['user'] = 'USR_ORG_' . $nrOrg;
                $connectionParams['password'] = $connectionParams['vpdPassword'];
                if(isset($connectionParams['isWallet']) && $connectionParams['isWallet'] == 'true') {
                    $connectionParams['dbname'] = isset($connectionParams['vpdDBNAME'])
                        ? $connectionParams['vpdDBNAME']
                        : $connectionParams['vpdDBName'];
                    $connectionParams['dbname'] = $connectionParams['vpdDBNAME'];
                    $connectionParams['host'] = $connectionParams['vpdHost'];
                    $connectionParams['port'] = $connectionParams['vpdPort'];
                    $connectionParams['service'] = $connectionParams['vpdService'];
                    unset($connectionParams['sessionMode']);
                }
            }
        }

        if(empty($connectionParams["password"])) {
            $connectionParams["password"] = '';
        } else if(isset($connectionParams["isEncrypted"]) && $connectionParams["isEncrypted"] == 'true') {
            $connectionParams["password"] = self::decrypt($connectionParams["password"], $connectionParams["cryptKey"]);
            $connectionParams["isEncrypted"] = false;
        }
        $conexao = DriverManager::getConnection($connectionParams, $config, $eventManager);
            
        return $conexao;
    }
    
    public static function returnEntity($connectionParams, $config, $eventManager){
        $conection = self::updateConnectionParams($connectionParams, $config, $eventManager);
        return EntityManager::create($conection, $config, $eventManager);
    }
}