describe("MetaDataFactory", function () {
	var previousZeptoPhoneValue = Zepto.os.phone;
	var localStorageEngine, requestDataSourceEngine, SQLStorageStrategy, restEngine = {
		requestMetaData: function () {
			return Q.when({});
		}
	};
	
	/*jslint evil: true */
	var eventEngine = new EventEngine(new EventFactory(new MethodBuilder({}, {
		$eval: function (a, b) {
			return eval(a, b);
		}
	}), {}), new EventHandler({}, {}, console), {
		updateTemplate: function () {}
	}, {
		i18n: function (text) {
			return text;
		}
	});
	
	var aclEngine = new AclEngine();
	
	var screenService = {
		isPopupOpen: function () {
			return false;
		}
	};
	
	var baseWidget = function() {
		return {
			"events": [{
				"name": "WidgetBeforeSelectAll",
				"code": ";"
			}, {
				"name": "WidgetAfterSelectAll",
				"code": ";"
			}],
			"dataSource": {
				"name": "/test",
				"sql": true,
				"data": [{}, {}, {}, {}]
			},
			"fields": [{
				"name": "name",
				"repeatOnNew": true,
				"defaultValue": 'nobody'
			}, {
				"name": "age",
				"repeatOnNew": false,
			}, {
				"name": "title",
				"repeatOnNew": true,
				"defaultValue": "Title d",
			}],
			"active": true,
			"actions": [{
				"name": "teste",
				"showAsAction": "edit_edit"
			}, {
				"name": "teste",
				"showAsAction": "edit"
			}, {
				"name": "teste",
				"showAsAction": "never"
			}, {
				"name": "teste",
				"showAsAction": "never|edit"
			}]
		};
	};
	
	var baseContainer = function () {
		return {
			"id": "20169762992902656303854",
			"name": "containator",
			"widgets": [{
				"dataSource": {
					"name": "/test2",
					"localStorage": true,
					"data": [{}, {}, {}, {}, {}]
				},
				"fields": [],
				"active": true,
				"widgets": [{
					"fields": [],
					"active": true,
					"dataSource": {
						"name": "/test2",
						"rest": true
					}
				}],
				"actions": [{
					"showAsAction": "checked_rows"
				}]
			}, ]
		};
	};
	
	var metaDataFactory = new MetaDataFactory(localStorageEngine, requestDataSourceEngine,
		SQLStorageStrategy, restEngine, eventEngine, screenService, aclEngine, {
			project: {}
		}, new ZhUtil({}), Q, {
			getDataSourceLazy: function () {}
		}, {}, {}, {}, {}, {}, {});
	
	it("Method projectFactory should just return the project the way it is if it's already initialized", function () {
		var baseProject = ZhHelper.projectTest();
			
		var project = metaDataFactory.projectFactory(baseProject);
		var secondProject = metaDataFactory.projectFactory(project);
			
		expect(secondProject).toBe(project);
	});
	
	it("Method widgetFactory should set the widge and it's children widgets container to the parent, if provided", function () {
		var baseProject = ZhHelper.projectTest();
			
		var widget = baseWidget();
		widget.widgets = [
			{
				name: "childWidget"
			}
		];
		var container = metaDataFactory.containerFactory(baseContainer());
		
		var factoredWidget = metaDataFactory.widgetFactory(widget, container);
		
			
		expect(factoredWidget.container).toBe(container);
		expect(factoredWidget.widgets[0].name).toBe("childWidget");
		expect(factoredWidget.widgets[0].container).toBe(container);
	});
	
	describe("MetaDataFactory (mobile OS)", function() {
		var widget = null;
		
		beforeEach(function () {
			Zepto.os.phone = true;
			
			var container = metaDataFactory.containerFactory(baseContainer());
			widget = baseWidget();
			widget.container = container;
		});
		
		afterEach(function() {
			Zepto.os.phone = previousZeptoPhoneValue;
		});
		
		it("Method widgetFactory should add mobile properties and replace array indexes on inheritance", function () {
			var mobileEvents = [
				{
					"name": "WidgetAfterMoveRow",
					"code": ";"
				}, 
				{
					"name": "WidgetOnEnter",
					"code": ";"
				},
				{
					"name": "WidgetOnDashboardClick",
					"code": ";"
				}
			];
			widget.mobile = {
				events: mobileEvents 
			};
			widget = metaDataFactory.widgetFactory(widget);
			
			expect(widget.events).toEqual(mobileEvents);
		});
		
		it("Method widgetFactory should replace by mobile properties if replaceByMobile is true", function () {
			widget.replaceByMobile = true;
			widget.events = [
				{
					"name": "WidgetAfterMoveRow",
					"code": "something awesome"
				},
				{
					"name": "WidgetAfterMoveRow",
					"code": "something awesome"
				},
				{
					"name": "WidgetAfterMoveRow",
					"code": "something awesome"
				}
			];
			var mobileEvents = [
				{
					"name": "WidgetOnEnter",
					"code": ";"
				}, 
				{
					"name": "WidgetOnEnter",
					"code": ";"
				}
			];
			widget.mobile = {
				events: mobileEvents 
			};
			
			widget = metaDataFactory.widgetFactory(widget);
			
			expect(widget.events).toEqual(mobileEvents);
		});
		
		it("Method widgetFactory should replace some object properties by mobile ones and the others should be unchanged", function () {
			widget.dataSource = {
				"name": "/test2",
				"localStorage": true,
				"data": [{}, {}, {}, {}, {}],
				"myDataSourceProperty": [{uaba: "fet"}]
			};
			widget.mobile = {
				dataSource: {
					"name": "/test3"
				}
			};
			
			widget = metaDataFactory.widgetFactory(widget);
			
			expect(widget.dataSource.name).toBe("/test3");
			expect(widget.dataSource.localStorage).toBe(true);
			expect(widget.dataSource.myDataSourceProperty).toEqual([{uaba: "fet"}]);
		});
		
		it("Method fieldFactory should replace some object properties by mobile ones", function () {
			var field = {
				name: "changedName",
				repeatOnNew: true,
				defaultValue: 'nobody',
				mobile: {
					name: "mobileName"
				}
			};
			
			widget = metaDataFactory.widgetFactory(widget);
			var previousWidgetFieldsLength = widget.fields.length;
			var newField = metaDataFactory.fieldFactory(field, widget, true);
			
			expect(widget.fields.length).toBe(previousWidgetFieldsLength + 1);
			expect(newField.name).toEqual("mobileName");
		});
		
		it("Method projectFactory should inherit some mobile properties", function () {
			var baseProject = ZhHelper.projectTest();
			
			var mobileNotifications = [
				{
					id: 2,
					name: "replace"
				},
				{
					id: 3,
					name: "replace2" 
				}, 
				{
					id: 4,
					name: "newNotification"
				}
			];
			baseProject.zeedhi_project.mobile = {
				notifications: mobileNotifications
			};
			var project = metaDataFactory.projectFactory(baseProject.zeedhi_project);
			
			expect(project.notifications[0].id).toEqual(mobileNotifications[0].id);
			expect(project.notifications[0].name).toEqual(mobileNotifications[0].name);
			expect(project.notifications[1].id).toEqual(mobileNotifications[1].id);
			expect(project.notifications[1].name).toEqual(mobileNotifications[1].name);
			expect(project.notifications[2].id).toEqual(mobileNotifications[2].id);
			expect(project.notifications[2].name).toEqual(mobileNotifications[2].name);
		});
		
		it("Method projectFactory should a notification if it has a mobile property", function () {
			var baseProject = ZhHelper.projectTest();
			
			baseProject.zeedhi_project.notifications = [
				{
					id: 2,
					name: "replace",
					mobile: {
						id: 3,
						name: "nowItsDifferent"
					}
				},
				{
					id: 3,
					name: "foo" 
				},
				{
					id: 4,
					name: "bar"
				}
			];
			var project = metaDataFactory.projectFactory(baseProject.zeedhi_project);
			
			expect(project.notifications.length).toEqual(3);
			expect(project.notifications[0].id).toEqual(3);
			expect(project.notifications[0].name).toEqual("nowItsDifferent");
		});
		
		it("Method containerFactory should inherit some mobile properties", function () {
			var container = baseContainer();
			
			container.mobile = {
				name: "changed"
			};
			newContainer = metaDataFactory.containerFactory(container);
			
			expect(newContainer.name).toEqual("changed");
			expect(newContainer.id).toEqual("20169762992902656303854");
		});
		
		it("Method widgetFactory inherit actions properties from mobile ones", function () {
			var mobileActions = [
				{
					"name": "act",
					"isVisible": true,
					"icon": "plus",
					"showAsAction": "grid-right",
					"events": [
						{
							"name": "ActionEvent",
							"code": "ScreenService.showMessage('NEW It works!')",
							"id": "13027869916792678961649"
						}
					],
					"id": "13027869918469122751648"
				},
				{
					"name": "willBeEdited",
					"isVisible": true,
					"icon": "plus",
					"showAsAction": "grid-right",
					"condition": "row.STATUS == 'I'",
					"id": "13027869918469122751648",
					"events": [
						{
							"name": "ActionEvent",
							"code": "ScreenService.showMessage('NEW It works!')",
							"id": "13027869916792678961649"
						}
					],
				},
			];
			widget.actions = mobileActions;
			
			widget.actions[1].mobile = {
				name: "isReplaced",
				isVisible: false,
				icon: "WOW"
			};
			
			widget = metaDataFactory.widgetFactory(widget);
			
			expect(widget.actions.length).toBe(2);
			expect(widget.actions[0].name).toEqual(mobileActions[0].name);
			expect(widget.actions[0].isVisible).toEqual(mobileActions[0].isVisible);
			expect(widget.actions[0].icon).toEqual(mobileActions[0].icon);
			expect(widget.actions[0].showAsAction).toEqual(mobileActions[0].showAsAction);
			expect(widget.actions[0].id).toEqual(mobileActions[0].id);
			expect(widget.actions[0].events).toEqual(mobileActions[0].events);
			
			expect(widget.actions[1].name).toBe("isReplaced");
			expect(widget.actions[1].isVisible).toBe(false);
			expect(widget.actions[1].icon).toBe("WOW");
			expect(widget.actions[1].events).toEqual(mobileActions[1].events);
		});
	});
});