describe("SelectController", function() {
	var selectController;
	var $scope;
	var $rootScope;
	var container;
	var widget;
	var field;

	window.templateUrls = [{
		package: "default",
		baseUrl: "../../../frontend/templates/dyman/"
	}];

	var baseContainer = ZhHelper.baseContainerWithTwoWidgets;
	var baseFields = function() {
		return [{
			name: "name",
			isVisible: true,
			aclIsVisible: true,
			showOnList: true,
			skipSearch: false,

		}, {
			name: "birth",
			isVisible: true,
			aclIsVisible: true,
			showOnList: true,
			skipSearch: false,
		}];
	};

	beforeEach(function() {
		module(function($provide) {
			//Providing a new $timeout because $timeout.flush is not flushing pending $timeout calls
			$provide.constant("$timeout", setTimeout);
			$provide.constant("$q", Q);
		});
		module("ZeedhiApp");
		module("ZeedhiServices");
		module('ZeedhiDirectives');


		inject(function(_$rootScope_, $controller, metaDataFactory, ScreenService) {
			screenService = ScreenService;
			container = baseContainer();
			container.widgets[0].template = "widget/form.html";
			container.widgets[0].fields = baseFields();
			container.widgets[0].fields[0].template = "field/select.html";
			container = metaDataFactory.containerFactory(container);
			window.metaDataFactory = metaDataFactory;

			widget = container.widgets[0];
			$rootScope = _$rootScope_;
			$scope = _$rootScope_.$new();
			$scope.$parent = _$rootScope_;
			$scope.widget = widget;
			$scope.field = widget.fields[0];
			field = $scope.field;
			field.dataSource = {
				clearCheckedRows: function() {},
				addCheckedRows: function() {}
			};
			field.element = {
				blur: function() {},
				focus: function() {},
				clearCheckedRows: function() {}
			};
			selectController = $controller("SelectController", {
				$scope: $scope,
				metaDataFactory: metaDataFactory,
				$rootScope: $rootScope
			});
		});
	});

	it("$scope.openField: should do nothing if the field is readOnly", function() {
		field.readOnly = true;
		spyOn(screenService, "openPopup").and.callThrough();
		$scope.openField(field);
		expect(screenService.openPopup).not.toHaveBeenCalled();
		//expect(selectMultipleController.getSelectWidget()).toBeUndefined();
	});

	it("$scope.openField: should create a widget and open it as a popup", function() {
		spyOn(screenService, "openPopup").and.callThrough();
		$scope.openField(field);
		expect(screenService.openPopup).toHaveBeenCalled();
		expect(field.selectWidget).not.toBeUndefined();
	});

	it("selectWidget.setSelected: should set the widget currentRow and call field's change event", function() {
		field.valueField = 'ID';
		field.descriptionField = 'NAME';
		var args = {owner:{selectField: field},row:{ID: 1, NAME: 'John'}};
		field.change = function() {};
		spyOn(field, "change").and.callThrough();
		$scope.openField(field);
		field.selectWidget.setSelected(args);
		expect(field.change).toHaveBeenCalled();
		expect(field.widget.currentRow[field.name]).toEqual('John');
		expect(field.widget.currentRow[field.valueField]).toEqual(1);
	});	
		
	it('the widget styles should have the parent field styles', function() {
		field.selectWidgetStyles = [{
			condition: true,
			cssClass: "zh-true"
		}];
		$scope.openField(field);
		expect(field.selectWidget.styles).toBe(field.selectWidgetStyles);
	});

});