describe("WidgetValidation", function() {
	var localStorageEngine, requestDataSourceEngine, SQLStorageStrategy, restEngine = {
		requestMetaData: function() {
			return Q.when({});
		}
	};
	var validateFunction;

	var widget = null;
	var eventHandler = new EventHandler({}, {}, console);
	var methodBuilder = new MethodBuilder({}, {
		$eval: function(a, b) {}
	});

	var eventEngine = new EventEngine(new EventFactory(methodBuilder, {}),
		eventHandler, {
			updateTemplate: function() {}
		}, {
			i18n: function(text) {
				return text;
			}
		}, {
			logEvent: function() {}
		}
	);

	var aclEngine = new AclEngine();
	var screenService = {
		isPopupOpen: function() {
			return false;
		},
		closeSwipe: function() {}
	};

	var injector = {
		get: function () {
			return {
				loadMessagesFile: function(pack) {
					var response;

					switch(pack){
						case 'default':
							response = Q.when({});
							break;
						case 'zeedhi':
							response = Q.when({
								"IS_VALID_WITH_MESSAGE": "Error in field '__fieldLabel__' with message: __errorMessage__.\n"
							});
							break;
					}

					return response;
				}
			};
		}
	};
	var validateEngine = {
		getMappedValidations: function() {
			return {
				required: function(value, validationAttrs) {
					var isDefined = !!value || Util.isNumber(value);
					return {
						"valid": !!(isDefined && (!Array.isArray(value) || value.length > 0)),
						"message": validationAttrs.message || "Required field"
					};
				}
			};
		}
	};

	var timeout = function(callback) {
		callback();
	};

	var messageService = new MessageService( screenService, { ZEEDHI_PACKAGE: 'zeedhi'}, injector, timeout);

	var metaDataFactory = new MetaDataFactory(localStorageEngine, requestDataSourceEngine,
		SQLStorageStrategy, restEngine, eventEngine, screenService, aclEngine, {
			project: {}
		}, new ZhUtil({}), Q, {
			getDataSourceLazy: function() {}
		}, {}, {}, {
			registerFeatureUsage: function() {}
		}, {}, validateEngine, messageService);

	var baseContainer = function() {
		return {
			"widgets": [{
				"events": [{
					"name": "WidgetBeforeSelectAll",
					"code": ";"
				}, {
					"name": "WidgetAfterSelectAll",
					"code": ";"
				}],
				"dataSource": {
					"name": "/test",
					"sql": true,
					"data": [{}, {}, {}, {}]
				},
				"fields": [{
					"name": "name",
					"repeatOnNew": true,
					"defaultValue": 'nobody'
				}, {
					"name": "age",
					"repeatOnNew": false,
				}, {
					"name": "title",
					"repeatOnNew": true,
					"defaultValue": "Title d",
				}, {
					"name": "validate",
					"label": "Field with required",
					"repeatOnNew": true,
					"defaultValue": "Title d",
					"validations": {
                        "required": true
                    }
				}],
				"active": true,
				"actions": [{
					"name": "teste",
					"showAsAction": "edit_edit"
				}, {
					"name": "teste",
					"showAsAction": "edit"
				}, {
					"name": "teste",
					"showAsAction": "never"
				}, {
					"name": "teste",
					"showAsAction": "never|edit"
				}]
			}, {
				"dataSource": {
					"name": "/test2",
					"localStorage": true,
					"data": [{}, {}, {}, {}, {}]
				},
				"fields": [],
				"active": true,
				"widgets": [{
					"fields": [],
					"active": true,
					"dataSource": {
						"name": "/test2",
						"rest": true
					}
				}],
				"actions": [{
					"showAsAction": "checked_rows"
				}]
			}]
		};
	};

	beforeEach(function() {
		container = metaDataFactory.containerFactory(baseContainer());
		widget = container.widgets[0];
		widget.restoreDefaultMode();
	});

	it("Should test if widget is valid with names and boolean return", function(done){
		var invalidFields;
		var callback = function(response){
			expect(response[3][0].errorMessage).toEqual("Required field");
			done();
		};
		expect(widget.isValid(callback)).toBe(false);
	});

	it("Should test if widget is valid with message return", function(done){
		var invalidFields;
		var callback = function(response, time, params){
			expect(response).toEqual("Error in field '__fieldLabel__' with message: __errorMessage__.\n");
			expect(params.fieldLabel).toBe('Field with required');
			expect(params.errorMessage).toBe('Required field');
			done();
		};

		widget.isValidWithMessage(callback);
	});
});