var MapNutritionistService = function(PatientsRepository, BedsService, ZHPromise, Validation, localStorageEngine) {

    var DISCHARGE = 0;
    var DEATH = 1;

    this.initDataSource = function() {

        return PatientsRepository.saveInServer().then(function() {
            return PatientsRepository.initDataSource();
        });

    };

    this.getPatientById = function(patientId) {
        return PatientsRepository.getPatientById(patientId).then(function(patient) {
            return patient;
        });
    };

    var _patientSituation = function(checkedPatients, situation) {

        var patients = [];
        var promises = checkedPatients.map(function(patient) {
            return PatientsRepository.getPatientById(patient.id)
                .then(function(patient) {

                    patient.dischargeDate = Validation.now();
                    patient.situation = situation;
                    patient.changed = true;
                    patient.oldSituation = situation;
                    if (situation === DEATH) {
                        patient.isDead = true;
                    }
                    patients.push(patient);

                });

        });

        var currentPatient = 0;

        var updateCurrentPatient = function() {

            if (currentPatient < patients.length) {
                BedsService.getBed(patients[currentPatient].bed).then(function(bed) {
                    patients[currentPatient].oldBed = bed;
                });
                return PatientsRepository.save(patients[currentPatient]).then(function() {
                    currentPatient++;
                    return updateCurrentPatient();
                });

            }
        };

        return ZHPromise.all(promises).then(function() {
            return updateCurrentPatient();
        });

    };

    this.confirmationDischarge = function(patientsDischarge) {
        return _patientSituation(patientsDischarge, DISCHARGE);
    };

    this.confirmationDeath = function(patientsDeath) {
        return _patientSituation(patientsDeath, DEATH);
    };

    this.confirmationPatientStatus = function(patientData) {
        return _patientSituation(patientData.patient, patientData.situation);
    };

    this.savePrescription = function(rowsToSave) {

        return localStorageEngine.save("/ControlFeedings", rowsToSave);

    };

    this.saveOralPrescriptionNormalFeedings = function(rowsToSave, changed) {
        if (changed === false) {
            if (Array.isArray(rowsToSave)) {
                rowsToSave.map(function(row) {
                    row.changed = false;
                    row.__is_new = false;
                });
            }
        }
        return localStorageEngine.save("/NormalFeedings", rowsToSave);
    };

    this.saveLactaryFeeding = function(rowsToSave, changed) {
        if (changed === false) {
            if (Array.isArray(rowsToSave)) {
                rowsToSave.map(function(row) {
                    row.changed = false;
                    row.__is_new = false;
                });
            }
        }
        return localStorageEngine.save("/LactaryFeedings", rowsToSave);

    };

    this.saveEnteralFeeding = function(rowsToSave) {

        localStorageEngine.save("/EnteralFeedings", rowsToSave);

    };

    this.savePatient = function(rowsToSave) {
        return localStorageEngine.save("/Patients", rowsToSave);
    };

};
Configuration(function(ContextRegister) {
    ContextRegister.register('MapNutritionistService', MapNutritionistService);
});
