var SynchronizeService = function(restEngine, $window, localStorageEngine, StructuresRepository, PatientsRepository, ZHPromise, ScreenService) {

    var _initStructures = function(unitID, widget) {
        var filterFunc = function(data) {
            return data.id === widget.curPatient.id;
        };
        var nutritionistID = getLocalVar('__CDOPERADOR');
        return restEngine.post('/Structures', {
            "requestType": "Row",
            "row": {
                "unitID": unitID,
                "nutritionistID": nutritionistID
            }
        }, null, null, null, null, 600000).then(function(success) {

            for (var key in success.dataset) {

                var name = '/' + key;
                var value = success.dataset[key];

                localStorageEngine.setDataSource(name, value);
                if (widget && widget.localDataSourceLoaded) {
                    var nameDataSourceLoaded = name.replace('/', '');
                    widget.localDataSourceLoaded[nameDataSourceLoaded] = angular.copy(value);
                    var dataToPatient = widget.localDataSourceLoaded[nameDataSourceLoaded].filter(filterFunc);
                    widget.curPatient[nameDataSourceLoaded] = dataToPatient;
                }

            }

        });

    };

    var _initPatients = function(unitID, widget) {
        var filterFunc = function(data) {
            return data.id === widget.curPatient.id;
        };
        var nutritionistID = getLocalVar('__CDOPERADOR');

        return restEngine.post('/Patients', {
            "requestType": "Row",
            "row": {
                "unitID": unitID,
                "nutritionistID": nutritionistID
            }
        }, null, null, null, null, 600000).then(function(success) {

            for (var key in success.dataset) {

                var name = '/' + key;
                var value = success.dataset[key];

                localStorageEngine.setDataSource(name, value);
                if (widget && widget.localDataSourceLoaded) {
                    var nameDataSourceLoaded = name.replace('/', '');
                    widget.localDataSourceLoaded[nameDataSourceLoaded] = angular.copy(value);
                    var dataToPatient = widget.localDataSourceLoaded[nameDataSourceLoaded].filter(filterFunc);
                    widget.curPatient[nameDataSourceLoaded] = dataToPatient;
                }
            }

        });
    };

    var _save = function(dataSet, dataSetsParentErrorDelete, conditionIdentification) {
        return localStorageEngine.getDataSource(dataSet).then(
            function success(rows) {
                var rowsToSave = [];
                var rowsToRemove = [];
                for (var i = 0; i < rows.length; i++) {
                    var row = rows[i];
                    if (!row.successToRemoveOnline) {
                        if (row.localChanges && row.localChanges.remove) {
                            rowsToRemove.push(row);
                        } else {
                            rowsToSave.push(row);
                        }
                    }
                }
                var promises = [];

                if (rowsToSave.length) {
                    promises.push(
                        localStorageEngine.saveInServer(dataSet, null, rowsToSave)
                    );
                }
                rowsToRemove = clearInconsistenciesToDelete(dataSetsParentErrorDelete, rowsToRemove, conditionIdentification);
                if (rowsToRemove.length) {
                    promises.push(
                        localStorageEngine.removeFromServer(dataSet, null, rowsToRemove)
                    );
                }

                return ZHPromise.all(promises);
            }
        );
    };

    function isValidArray(array) {
        return Array.isArray(array) && array.length;
    }

    function replaceAllStr(string, oldStr, newStr) {
        if (typeof string === 'string' && typeof oldStr === 'string' && typeof newStr === 'string') {
            return string.split(oldStr).join(newStr);
        }
        return string;
    }

    function clearInconsistenciesToDelete(dataSetsParentErrorDelete, dataSetChildren, conditionIdentification) {
        if (isValidArray(dataSetsParentErrorDelete) && isValidArray(dataSetChildren) && conditionIdentification && conditionIdentification.IDENTIFICATION) {
            conditionIdentification = conditionIdentification.IDENTIFICATION;
            var deletePropagatesDataSetChildren = [];
            deletePropagatesDataSetChildren = dataSetChildren.filter(function(children) {
                return !children.manualDelete;
            });
            if (isValidArray(deletePropagatesDataSetChildren)) {
                deletePropagatesDataSetChildren.filter(function(childrenToDelete) { // jshint ignore:line
                    var foundedRelateParentError;
                    try {
                        foundedRelateParentError = dataSetsParentErrorDelete.filter(function(errorToDeleteParent) { // jshint ignore:line
                            var bkpCondition = conditionIdentification;
                            bkpCondition = replaceAllStr(bkpCondition, 'PARENT', 'errorToDeleteParent');
                            bkpCondition = replaceAllStr(bkpCondition, 'CHILDREN', 'childrenToDelete');
                            return eval(bkpCondition); // jshint ignore:line
                        })[0];
                        if (foundedRelateParentError) {
                            childrenToDelete.inconsistenceToDelete = true;
                        }
                    } catch (error) {
                        return null;
                    }
                });
            }
        }
        return dataSetChildren;
    }

    var _get = function(widget) {
        var unitID = localStorageEngine.getLocalVar('__CDFILIAL');

        var promises = [];

        if (widget) {
            promises.push(_initStructures(unitID, widget));
            promises.push(_initPatients(unitID, widget));
        } else {
            promises.push(_initStructures(unitID));
            promises.push(_initPatients(unitID));
        }
        return ZHPromise.all(promises);

    };

    this.synchronizedIntolerance = function(widget, filter) {
        var patient = widget.curPatient;
        var deferred = ZHPromise.defer();
        syncRestriction(widget).then(function() {
            var arrayParamsProcedure = [];
            var filterRowPatient = widget.curPatient.ControlFeedings;
            if (filter) {
                filterRowPatient = widget.curPatient.ControlFeedings.filter(function(controlFeeding) {
                    return controlFeeding.id === filter.id && controlFeeding.serviceDate === filter.serviceDate && controlFeeding.DTINCLUSAO === filter.DTINCLUSAO;
                });
            }
            filterRowPatient.forEach(function(currentRow) {
                if (currentRow.idtpservhosp === '0' && currentRow.idformdiet !== '4') {
                    var paramsProcedure = {
                        'P_CDFILIAL': currentRow.unit,
                        'P_CDSERVFILI': currentRow.cdservfili,
                        'P_CDPACIENTE': patient.id,
                        'P_DTSERVINTE': currentRow.serviceDate,
                        'P_NRSEQUSERV': currentRow.sequential,
                        'P_DTINTERNA': moment(patient.hospitalization, 'DD/MM/YYYY HH:mm:ss').format('DD/MM/YYYY')
                    };
                    arrayParamsProcedure.push(paramsProcedure);
                }
            });
            if (isValidArray(arrayParamsProcedure)) {
                restEngine.post('/Patients/updateFoodRestriction', {
                    "requestType": "DataSet",
                    "dataset": arrayParamsProcedure
                }, null, null, null, null, 500000).then(function() {
                    _get(widget).then(function() {
                        deferred.resolve(true);
                    }, function() {
                        deferred.reject(false);
                    });
                }, function() {
                    deferred.reject(false);
                });
            } else {
                deferred.resolve(true);
            }
        }, function() {
            deferred.reject(false);
        });
        return deferred.promise;
    };

    function verifyOccurredErrorCallbackArray(callback) {
        var errorsHaveOccurred = false;
        if (typeof callback === 'object') {
            var objectCallback = Object.entries(callback);
            if (isValidArray(objectCallback)) {
                objectCallback.forEach(function(dataSourceError) {
                    if (isValidArrayOfError(dataSourceError[1], dataSourceError[0])) {
                        errorsHaveOccurred = true;
                    }
                });
            }
        }
        return errorsHaveOccurred;
    }

    function syncRestriction(widget) {
        var deferred = ZHPromise.defer();
        var relationshipId = getIdentificationOfRelationship();
        return _save('/ControlFeedings').then(function(CONTROL_FEEDING) {
            var arrayErrorControl = getArrayOfError(CONTROL_FEEDING, 'CONTROL_FEEDING', 'DELETE');
            return _save('/Patients').then(function(PATIENT) {
                return _save('/Profiles').then(function(PROFILE) {
                    return _save('/NormalFeedings', arrayErrorControl, relationshipId.CONTROL_FEEDING).then(function(NORMALFEEDING) {
                        return _save('/FoodIntoleranceGroup').then(function(FOODINTOLERANCEGROUP) {
                            var callback = {
                                'CONTROL_FEEDING': CONTROL_FEEDING,
                                'PATIENT': PATIENT,
                                'PROFILE': PROFILE,
                                'NORMALFEEDING': NORMALFEEDING,
                                'FOODINTOLERANCEGROUP': FOODINTOLERANCEGROUP
                            };
                            var error = verifyOccurredErrorCallbackArray(callback);
                            if (!error) {
                                _get(widget).then(function() {
                                    deferred.resolve();
                                });
                            } else {
                                deferred.reject();
                            }
                            return deferred;
                        });
                    });
                });
            });
        });
    }

    function getMessageInfo(typeStruct, arrayInfo, propertyToDisplay) {
        var msg = "<p class='titleStructError'>" + i18n(typeStruct) + ':</p><br />';
        var subcategory = {};
        arrayInfo.forEach(function(currentInfo, indexInfo) {
            var message = '';
            if (currentInfo.SUBCATEGORY) {
                if (!subcategory[currentInfo.SUBCATEGORY]) {
                    subcategory[currentInfo.SUBCATEGORY] = true;
                    message = message.concat("<p class='subCategory'>" + i18n(currentInfo.SUBCATEGORYNAME) + "</p><br />");
                }
            }
            if (currentInfo.id) {
                message = message.concat(i18n('Código Paciente: ') + currentInfo.id + "<br />");
            }
            if (currentInfo.GENERIC_MESSAGE) {
                message = message.concat(i18n(currentInfo.GENERIC_MESSAGE));
            } else {
                propertyToDisplay.forEach(function(property, index) {
                    message = message.concat(i18n(currentInfo[property]));
                    if (index !== propertyToDisplay.length - 1) {
                        message = message.concat(' - ');
                    }
                });
            }
            if (indexInfo < arrayInfo.length - 1) {
                message = message.concat('<br />');
            }
            if (msg.indexOf(message) === -1) {
                msg = msg.concat(message);
            }
        });
        return msg;
    }

    function isValidArrayOfError(callback, nameArray) {
        return Array.isArray(callback) && callback.length && callback[0].dataset && callback[0].dataset.error && Array.isArray(callback[0].dataset.error[nameArray]) && callback[0].dataset.error[nameArray].length;
    }

    function i18n(string) {
        if (typeof string === 'string') {
            return ScreenService.i18n(string);
        }
        return "";
    }

    function concatCallbackError(callback, nameArray) {
        var fullCallback;
        if (isValidArray(callback)) {
            callback.forEach(function(currentCallback) {
                if (isValidArrayOfError([currentCallback], nameArray)) {
                    if (!fullCallback) {
                        fullCallback = [angular.copy(currentCallback)];
                    } else {
                        fullCallback[0].dataset.error[nameArray] = fullCallback[0].dataset.error[nameArray].concat(currentCallback.dataset.error[nameArray]);
                    }
                }
            });
            return fullCallback;
        }
        return [];
    }

    function isValidObjectError(object) {
        return object && object.dataset && object.dataset.error;
    }

    function getArrayOfError(callback, nameArray, typeOperation) {
        if (isValidArray(callback)) {
            if (typeOperation && typeof typeOperation === 'string') {
                var filtredCallback = callback.filter(function(data) {
                    if (isValidArray(data) || isValidObjectError(data)) {
                        return data.dataset.error.OPERATION === typeOperation;
                    } else {
                        return false;
                    }
                })[0];
                if (filtredCallback) {
                    return filtredCallback.dataset.error[nameArray];
                }
            } else if (isValidArrayOfError(callback, nameArray)) {
                return callback[0].dataset.error[nameArray];
            }
        }
        return [];
    }

    function errorHandling(typeStruct, callback, nameArray, propertyToDisplay, referenceMainMessage) {
        var normalizedCallBack = concatCallbackError(callback, nameArray);
        if (isValidArrayOfError(normalizedCallBack, nameArray)) {
            var struct = ScreenService.i18n(typeStruct);
            var arrayOfError = getArrayOfError(normalizedCallBack, nameArray);
            var message = getMessageInfo(struct, arrayOfError, propertyToDisplay);
            if (referenceMainMessage.numberOfErrors === 0) {
                referenceMainMessage.msg = referenceMainMessage.msg.concat('<br /><br />');
            }
            if (referenceMainMessage.numberOfErrors > 0) {
                referenceMainMessage.msg = referenceMainMessage.msg.concat("<hr class='lineWrap'>");
            }
            referenceMainMessage.msg = referenceMainMessage.msg.concat(message);
            referenceMainMessage.msg = referenceMainMessage.msg.concat('<br />');
            referenceMainMessage.numberOfErrors++;
            return true;
        }
        return false;
    }

    function getStructureDataError() {
        var dataSets = {
            '/ControlFeedings': {
                'name': 'CONTROL_FEEDING',
                'typeData': 'Prescrições do Controle Alimentar',
                'propertyToDisplay': ['serviceDate', 'serviceName', 'dietName']
            },
            '/User': {
                'name': 'USER',
                'typeData': 'Usuários',
                'propertyToDisplay': []
            },
            '/Patients': {
                'name': 'PATIENT',
                'typeData': 'Pacientes',
                'propertyToDisplay': []
            },
            '/Profiles': {
                'name': 'PROFILE',
                'typeData': 'Perfil Alimentar',
                'propertyToDisplay': ['nmgroupsub'],
            },
            '/Prescriptions': {
                'name': 'PRESCRIPTION',
                'typeData': 'Prescrições Dietéticas',
                'propertyToDisplay': ['date', 'dietName', 'bed']
            },
            '/Appointments': {
                'name': 'APPOINTMENT',
                'typeData': 'Consultas',
                'propertyToDisplay': []
            },
            '/BioEvaluations': {
                'name': 'BIOEVALUATION',
                'typeData': 'Avaliações Bioquímicas',
                'propertyToDisplay': []
            },
            '/NormalFeedings': {
                'name': 'NORMALFEEDING',
                'typeData': 'Pratos Prescrição Oral',
                'propertyToDisplay': ['serviceDate', 'plateTypeName', 'plateName']
            },
            '/EnteralFeedings': {
                'name': 'ENTERALFEEDING',
                'typeData': 'Pratos Prescrição Enteral',
                'propertyToDisplay': []
            },
            '/LactaryFeedings': {
                'name': 'LACTARYFEEDING',
                'typeData': 'Pratos Prescrição Lactária',
                'propertyToDisplay': ['serviceDate', 'formulationName']
            },
            '/FoodIntoleranceGroup': {
                'name': 'FOODINTOLERANCEGROUP',
                'typeData': 'Grupos de Restrição Alimentar',
                'propertyToDisplay': []
            }
        };
        return dataSets;
    }

    this.persistOnline = function(dataSourceName, route, type, param, timeout) {
        var isOnline = localStorageEngine.getLocalVar('__ONLINE');
        var defer = ZHPromise.defer();
        if (isOnline === true) {
            if (typeof route === 'string' && typeof type === 'string') {
                var typeLow = type.toLowerCase();
                var objectRequest = {
                    requestType: type
                };
                objectRequest[typeLow] = param;
                restEngine.post(route, objectRequest, null, null, null, null, timeout).then(function(callback) {
                    if (isValidArrayOfError(callback, dataSourceName)) {
                        var structure = getStructureDataError();
                        var currentStructure = structure[dataSourceName];
                        var msgError = {
                            'msg': ScreenService.i18n("Ocorreram erros ao salvar alterações:"),
                            'numberOfErrors': 0
                        };
                        errorHandling(currentStructure.typeData, callback, currentStructure.name, currentStructure.propertyToDisplay, msgError);
                        ScreenService.showMessage(msgError.msg);
                        defer.reject(false);
                    }
                    defer.resolve(callback);
                    return defer;

                }, function() {
                    ScreenService.showMessage(i18n('Falha na conexão. Sincronize para salvar dados locais.'));
                    defer.reject(null);
                });

            } else {
                defer.reject(null);
            }
        } else {
            defer.reject(null);
        }
        return defer;
    };

    function crudPromisse(dataSources, typeOperation) {
        var promise = [];
        if (typeOperation === 'clear', isValidArray(dataSources)) {
            dataSources.forEach(function(dataSource) {
                promise.push(
                    localStorageEngine.setDataSource(dataSource, [])
                );
            });
        }
        return ZHPromise.all(promise);
    }

    function getIdentificationOfRelationship() {
        var parentsAndId = {
            'CONTROL_FEEDING': {
                'IDENTIFICATION': 'PARENT.nrsequserv === CHILDREN.sequential && PARENT.id === CHILDREN.id'
            }
        };
        return parentsAndId;
    }
    this.sync = function(changedUnit) {
        var msgError = {
            'msg': ScreenService.i18n("Ocorreram erros ao sincronizar os seguintes dados:"),
            'numberOfErrors': 0
        };
        var relationshipId = getIdentificationOfRelationship();
        var errorsHaveOccurred = false;
        return _save('/User').then(function(USER) {
            return _save('/Patients').then(function(PATIENT) {
                return _save('/Profiles').then(function(PROFILE) {
                    return _save('/ControlFeedings').then(function(CONTROL_FEEDING) {
                        var arrayErrorControl = getArrayOfError(CONTROL_FEEDING, 'CONTROL_FEEDING', 'DELETE');
                        return _save('/Prescriptions').then(function(PRESCRIPTION) {
                            return _save('/Appointments').then(function(APPOINTMENT) {
                                return _save('/BioEvaluations').then(function(BIOEVALUATION) {
                                    return _save('/NormalFeedings', arrayErrorControl, relationshipId.CONTROL_FEEDING).then(function(NORMALFEEDING) {
                                        return _save('/EnteralFeedings', arrayErrorControl, relationshipId.CONTROL_FEEDING).then(function(ENTERALFEEDING) {
                                            return _save('/LactaryFeedings', arrayErrorControl, relationshipId.CONTROL_FEEDING).then(function(LACTARYFEEDING) {
                                                return _save('/FoodIntoleranceGroup').then(function(FOODINTOLERANCEGROUP) {
                                                    var deferred = ZHPromise.defer();
                                                    var dataSets = Object.values(getStructureDataError());
                                                    var callback = {
                                                        'CONTROL_FEEDING': CONTROL_FEEDING,
                                                        'USER': USER,
                                                        'PATIENT': PATIENT,
                                                        'PROFILE': PROFILE,
                                                        'PRESCRIPTION': PRESCRIPTION,
                                                        'APPOINTMENT': APPOINTMENT,
                                                        'BIOEVALUATION': BIOEVALUATION,
                                                        'NORMALFEEDING': NORMALFEEDING,
                                                        'ENTERALFEEDING': ENTERALFEEDING,
                                                        'LACTARYFEEDING': LACTARYFEEDING,
                                                        'FOODINTOLERANCEGROUP': FOODINTOLERANCEGROUP
                                                    };
                                                    dataSets.forEach(function(dataSet) {
                                                        var currentErro = errorHandling(dataSet.typeData, callback[dataSet.name], dataSet.name, dataSet.propertyToDisplay, msgError);
                                                        errorsHaveOccurred = errorsHaveOccurred ? true : currentErro;
                                                    });
                                                    if (changedUnit) {
                                                        var dataSource = ['/AccessRoads', '/Beds', '/DataEvaluations', '/ControlFeedings', '/DataTypeEvaluations', '/Examinations', '/Formulations', '/HealthInsurance', '/NutritionalStatus', '/Patients', '/Prescriptions', '/Profiles', '/Services', '/Appointments'];
                                                        crudPromisse(dataSource, 'clear').then(function() {
                                                            _get().then(function() {
                                                                if (errorsHaveOccurred) {
                                                                    ScreenService.showMessage(msgError.msg);
                                                                    deferred.resolve(true);
                                                                } else {
                                                                    deferred.resolve(false);
                                                                }
                                                            });
                                                        });
                                                    } else {
                                                        _get().then(function() {
                                                            if (errorsHaveOccurred) {
                                                                ScreenService.showMessage(msgError.msg);
                                                                deferred.resolve(true);
                                                            } else {
                                                                deferred.resolve(false);
                                                            }
                                                        });
                                                    }
                                                    return deferred;
                                                });

                                            });
                                        });
                                    });
                                });
                            });
                        });
                    });
                });
            });
        });
    };
};
Configuration(function(ContextRegister) {
    ContextRegister.register('SynchronizeService', SynchronizeService);
});
